'use strict';

exports.__esModule = true;

var _remoteProtocol = require('remote-protocol');

var _RemoteSession = require('./RemoteSession');

var _RemoteSession2 = _interopRequireDefault(_RemoteSession);

var _RemoteValue = require('./RemoteValue');

var _RemoteValue2 = _interopRequireDefault(_RemoteValue);

var _actions = require('./actions');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } } /**
                                                                                                                                                           * Copyright 2017 Moshe Simantov
                                                                                                                                                           *
                                                                                                                                                           * Licensed under the Apache License, Version 2.0 (the "License");
                                                                                                                                                           * you may not use this file except in compliance with the License.
                                                                                                                                                           * You may obtain a copy of the License at
                                                                                                                                                           *
                                                                                                                                                           *     http://www.apache.org/licenses/LICENSE-2.0
                                                                                                                                                           *
                                                                                                                                                           * Unless required by applicable law or agreed to in writing, software
                                                                                                                                                           * distributed under the License is distributed on an "AS IS" BASIS,
                                                                                                                                                           * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
                                                                                                                                                           * See the License for the specific language governing permissions and
                                                                                                                                                           * limitations under the License.
                                                                                                                                                           */

var kAction = Symbol('action');

/**
 * @example
 * const promise = new RemotePromise(remoteSession, action);
 *
 * // resolve the action value
 * promise.then(value => {
 *   // `value` is the resolve remote action value
 * });
 *
 * // resolve property of the action value
 * promise.find().then(value => {
 *   // `value` === resolveOnRemote(action.fetch().find())
 * });
 */

var RemotePromise =
/**
 * Create a remote promise from a given action.
 *
 * @param {RemoteSession} session The remote session
 * @param {Action} action The given action
 * @param {object} [opts] {@link Session#request} options
 * @return {function} The remote promise
 */
function RemotePromise(session, action) {
  var opts = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

  _classCallCheck(this, RemotePromise);

  if (!(session instanceof _RemoteSession2.default)) {
    throw new TypeError('Expect "session" param to be instance of RemoteSession: ' + session);
  }
  if (!(action instanceof _remoteProtocol.Action)) {
    throw new TypeError('Expect "action" param to be instance of Action: ' + action);
  }

  // eslint-disable-next-line func-names
  var func = function func() {
    throw new ReferenceError("This is a remote-promise function and it should'nt be execute");
  };

  func[kAction] = action;
  var promise = void 0;

  func.then = function then(onFulfilled, onRejected) {
    var _this = this;

    if (!promise) {
      promise = new Promise(function (resolve, reject) {
        var rejectOnSessionClose = function rejectOnSessionClose() {
          return reject(new Error('Session closed before promise resolved'));
        };

        session.request(_actions.ReflectPromiseAction.fromValue(session, _this[kAction], function (value) {
          session.removeListener('close', rejectOnSessionClose);

          _RemoteValue2.default.resolve(value).then(resolve, reject);
        }, function (error) {
          session.removeListener('close', rejectOnSessionClose);
          reject(error);
        }), opts, function () {
          session.once('close', rejectOnSessionClose);
        }, function (error) {
          reject(error);
        });
      });
    }

    return promise.then(onFulfilled, onRejected);
  };

  func.catch = function promiseCatch(onRejected) {
    return this.then(null, onRejected);
  };

  return new Proxy(func, {
    get: function get(target, property) {
      if (property in target) return target[property];

      return new RemotePromise(session, _actions.ReflectGetAction.fromProxy(session, action, property), opts);
    },
    apply: function apply(target, thisArg, argumentsList) {
      return new RemotePromise(session, _actions.ReflectApplyAction.fromProxy(session, action, thisArg, argumentsList), opts);
    },
    construct: function construct(target, argumentsList) {
      return new RemotePromise(session, _actions.ReflectConstructAction.fromProxy(session, action, argumentsList), opts);
    }
  });
};

exports.default = RemotePromise;